#include <OGRE/OgreVector3.h>
#include <OGRE/OgreSceneNode.h>
#include <OGRE/OgreSceneManager.h>

#include <rviz/ogre_helpers/arrow.h>
#include <geometry_msgs/Vector3.h>

#include "perception_visual.h"

namespace rviz_multiception
{

PerceptionVisual::PerceptionVisual(Ogre::SceneManager* scene_manager, Ogre::SceneNode* parent_node)
{
	scene_manager_ = scene_manager;

	// Ogre::SceneNode s form a tree, with each node storing the
	// transform (position and orientation) of itself relative to its
	// parent. Ogre does the math of combining those transforms when it
	// is time to render.
	//
	// Here we create a node to store the pose of the header frame
	// relative to the RViz fixed frame.
	frame_node_ = parent_node->createChildSceneNode();

	// We create the arrow object within the frame node so that we can
	// set its position and direction relative to its header frame.
	pose_arrow_.reset(new rviz::Arrow(scene_manager_, frame_node_));
}

PerceptionVisual::~PerceptionVisual()
{
	// Destroy the frame node since we don't need it anymore.
	scene_manager_->destroySceneNode(frame_node_);
}

void PerceptionVisual::setMessage(const multiception::Perception* msg)
{
	if(msg==nullptr) return;

	// Convert the geometry_msgs::Vector3 to an Ogre::Vector3.
	Ogre::Vector3 pose(msg->state.x, msg->state.y, 0);
	pose_arrow_->setPosition(pose);

	// Find the magnitude of the velocity vector.
	float length = 2;//msg->state.v;

	// Scale the arrow's thickness in each dimension along with its length.
	Ogre::Vector3 scale(length, length, length);
	pose_arrow_->setScale(scale);

	// Set the orientation of the arrow according to the heading.
	Ogre::Vector3 direction(std::cos(msg->state.yaw), std::sin(msg->state.yaw), 0);
	pose_arrow_->setDirection(direction);
}

// Position and orientation are passed through to the SceneNode.
void PerceptionVisual::setFramePosition(const Ogre::Vector3& position)
{
	frame_node_->setPosition(position);
}

void PerceptionVisual::setFrameOrientation(const Ogre::Quaternion& orientation)
{
	frame_node_->setOrientation(orientation);
}

// Color is passed through to the Arrow object.
void PerceptionVisual::setColor( float r, float g, float b, float a )
{
	pose_arrow_->setColor(r, g, b, a);
}

} // namespace rviz_multiception

