#include <OGRE/OgreSceneNode.h>
#include <OGRE/OgreSceneManager.h>

#include <tf/transform_listener.h>

#include <rviz/visualization_manager.h>
#include <rviz/properties/color_property.h>
#include <rviz/properties/float_property.h>
#include <rviz/default_plugin/covariance_property.h>
#include <rviz/properties/int_property.h>
#include <rviz/frame_manager.h>

#include "state_visual.h"
#include "state_display.h"

namespace rviz_multiception
{

// The constructor must have no arguments, so we can't give the
// constructor the parameters it needs to fully initialize.
StateDisplay::StateDisplay()
{
	color_property_ = new rviz::ColorProperty( "Color", QColor( 204, 51, 204 ),
	                                           "Color to draw the arrows.",
	                                           this, SLOT( updateColorAndAlpha() ));

	alpha_property_ = new rviz::FloatProperty( "Alpha", 1.0,
	                                           "Object transparency, 0 is fully transparent, 1.0 is fully opaque.",
	                                           this, SLOT( updateColorAndAlpha() ));

	covariance_property_ = new rviz::CovarianceProperty( "Covariance", true, "Whether or not the covariances of the messages should be shown.",
	                                                     this, SLOT( queueRender() ));
}

// After the top-level rviz::Display::initialize() does its own setup,
// it calls the subclass's onInitialize() function. This is where we
// instantiate all the workings of the class. We make sure to also
// call our immediate super-class's onInitialize() function, since it
// does important stuff setting up the message filter.
//
//  Note that "MFDClass" is a typedef of
// ``MessageFilterDisplay<message type>``, to save typing that long
// templated class name every time you need to refer to the
// superclass.
void StateDisplay::onInitialize()
{
	MFDClass::onInitialize();
}

StateDisplay::~StateDisplay()
{
}

// Clear the visuals by deleting their objects.
void StateDisplay::reset()
{
	MFDClass::reset();
	visual_.reset();
}

// Set the current color and alpha values for each visual.
void StateDisplay::updateColorAndAlpha()
{
	float alpha = alpha_property_->getFloat();
	Ogre::ColourValue color = color_property_->getOgreColor();
	visual_->setColor(color.r, color.g, color.b, alpha);
}

// This is our callback to handle an incoming message.
void StateDisplay::processMessage(const multiception::StateStamped::ConstPtr& msg)
{
	// Here we call the rviz::FrameManager to get the transform from the
	// fixed frame to the frame in the header of this message. If
	// it fails, we can't do anything else so we return.
	Ogre::Quaternion orientation;
	Ogre::Vector3 position;

	if(!context_->getFrameManager()->getTransform(msg->header.frame_id, msg->header.stamp, position, orientation))
	{
		ROS_DEBUG("Error transforming from frame '%s' to frame '%s'", msg->header.frame_id.c_str(), qPrintable(fixed_frame_));
		return;
	}

	visual_.reset(new StateVisual(context_->getSceneManager(), scene_node_, covariance_property_));

	// Now set or update the contents of the visual.
	visual_->setContent(msg->state);
	visual_->setFramePosition(position);
	visual_->setFrameOrientation(orientation);

	updateColorAndAlpha();
	context_->queueRender();
}

} // end namespace rviz_multiception

// Tell pluginlib about this class. It is important to do this in
// global scope, outside our package's namespace.
#include <pluginlib/class_list_macros.h>
PLUGINLIB_EXPORT_CLASS(rviz_multiception::StateDisplay,rviz::Display)
